import datetime

from django.db import models
from django.utils.translation import ugettext_lazy as _
from django.utils import timezone
from django.utils.crypto import get_random_string
from django.utils.encoding import python_2_unicode_compatible
from django.contrib.sites.models import Site
from django.core.urlresolvers import reverse

from allauth.account.adapter import DefaultAccountAdapter
from allauth.account.adapter import get_adapter

from .managers import InvitationManager
from . import app_settings
from . import signals


@python_2_unicode_compatible
class Invitation(models.Model):

    email = models.EmailField(unique=True, verbose_name=_('e-mail address'))
    accepted = models.BooleanField(verbose_name=_('accepted'), default=False)
    created = models.DateTimeField(verbose_name=_('created'),
                                   default=timezone.now)
    key = models.CharField(verbose_name=_('key'), max_length=64, unique=True)
    sent = models.DateTimeField(verbose_name=_('sent'), null=True)

    objects = InvitationManager()

    @classmethod
    def create(cls, email):
        key = get_random_string(64).lower()
        instance = cls._default_manager.create(
            email=email,
            key=key)
        return instance

    def key_expired(self):
        expiration_date = (
            self.sent + datetime.timedelta(
                days=app_settings.INVITATION_EXPIRY))
        return expiration_date <= timezone.now()
    key_expired.boolean = True

    def send_invitation(self, request, **kwargs):
        current_site = (kwargs['site'] if 'site' in kwargs
                        else Site.objects.get_current())
        invite_url = reverse('invitations:accept-invite',
                             args=[self.key])
        invite_url = request.build_absolute_uri(invite_url)

        ctx = {
            'invite_url': invite_url,
            'current_site': current_site,
            'email': self.email,
            'key': self.key,
        }

        email_template = 'invitations/email/email_invite'

        get_adapter().send_mail(email_template,
                                self.email,
                                ctx)
        self.sent = timezone.now()
        self.save()
        signals.invite_url_sent.send(
            sender=self.__class__,
            instance=self,
            invite_url_sent=invite_url)

    def __str__(self):
        return u"Invite: {}".format(self.email)


class InvitationsAdapter(DefaultAccountAdapter):

    def is_open_for_signup(self, request):
        if request.session.get('account_verified_email'):
            mail = request.session.get('account_verified_email')
            return True
        elif app_settings.INVITATION_ONLY is True:
            # Site is ONLY open for invites
            return False
        else:
            # Site is open to signup
            return True



class AppSettings(object):

    def __init__(self, prefix):
        self.prefix = prefix

    def _setting(self, name, dflt):
        from django.conf import settings
        return getattr(settings, self.prefix + name, dflt)

    @property
    def ALLOWED_GROUPS(self):
        """ Users in this group can send invites """
        return self._setting('ALLOWED_GROUPS', None)

    @property
    def INVITATION_EXPIRY(self):
        """ How long before the invitation expires """
        return self._setting('INVITATION_EXPIRY', 3)
    
    @property
    def INVITATION_ONLY(self):
        """ Signup is invite only """
        return self._setting('INVITATION_ONLY', False)

    @property
    def CONFIRM_INVITE_ON_GET(self):
        """ Simple get request confirms invite """
        return self._setting('CONFIRM_EMAIL_ON_GET', True)

    @property
    def SIGNUP_REDIRECT(self):
        """ Where to redirect on email confirm of invite """
        return self._setting('SIGNUP_REDIRECT', 'account_signup')


import sys
app_settings = AppSettings('INVITATIONS_')
app_settings.__name__ = __name__
sys.modules[__name__] = app_settings



from django import forms
from django.utils.translation import ugettext_lazy as _

from allauth.account.forms import AddEmailForm

from .models import Invitation


class InviteForm(AddEmailForm):

    def clean_email(self):
        value = super(InviteForm, self).clean_email()

        errors = {
            "already_invited": _("This e-mail address has already been"
                                 " invited."),
        }

        if Invitation.objects.filter(email__iexact=value,
                                     accepted=False):
            raise forms.ValidationError(errors["already_invited"])

        return value

    def save(self, email):
        return Invitation.create(email=email)



from django.views.generic import FormView, View
from django.views.generic.detail import SingleObjectMixin
from django.contrib import messages
from django.http import Http404
from django.shortcuts import redirect

from braces.views import LoginRequiredMixin, GroupRequiredMixin
from allauth.account.adapter import get_adapter

from .forms import InviteForm
from .models import Invitation
from . import app_settings, signals


class SendInvite(LoginRequiredMixin, GroupRequiredMixin, FormView):
    template_name = 'invitations/forms/_invite.html'
    form_class = InviteForm

    if app_settings.ALLOWED_GROUPS:
        group_required = app_settings.ALLOWED_GROUPS

    def form_valid(self, form):
        email = form.cleaned_data["email"]

        try:
            invite = form.save(email)
            invite.send_invitation(self.request)
        except Exception as e:
            return self.form_invalid(form)

        return self.render_to_response(
            self.get_context_data(
                success_message='%s has been invited' % email))

    def form_invalid(self, form):
        return self.render_to_response(self.get_context_data(form=form))


class AcceptInvite(SingleObjectMixin, View):
    form_class = InviteForm

    def get(self, *args, **kwargs):
        if app_settings.CONFIRM_INVITE_ON_GET:
            return self.post(*args, **kwargs)
        else:
            raise Http404()

    def post(self, *args, **kwargs):
        self.object = invitation = self.get_object()
        invitation.accepted = True
        invitation.save()
        get_adapter().stash_verified_email(self.request, invitation.email)
        
        signals.invite_accepted.send(sender=self.request.user.__class__,
                                     request=self.request,
                                     email=invitation.email)

        get_adapter().add_message(self.request,
                                  messages.SUCCESS,
                                  'invitations/messages/invite_accepted.txt',
                                  {'email': invitation.email})

        return redirect(app_settings.SIGNUP_REDIRECT)

    def get_object(self, queryset=None):
        if queryset is None:
            queryset = self.get_queryset()
        try:
            return queryset.get(key=self.kwargs["key"].lower())
        except Invitation.DoesNotExist:
            raise Http404()

    def get_queryset(self):
        return Invitation.objects.all_valid()



from django.dispatch import Signal

invite_url_sent = Signal(providing_args=['invite_url_sent'])
invite_accepted = Signal(providing_args=['email'])

"""
@receiver(invite_url_sent, sender=Invitation)
def invite_url_sent(sender, instance, invite_url_sent, **kwargs):
    pass

@receiver(invite_accepted, sender=auth.models.AnonymousUser)
def invite_accepted(sender, request, email, **kwargs):
    pass
"""


from django.conf.urls import patterns, include, url

from . import views

urlpatterns = patterns('',

    url(r'^send-invite/$', views.SendInvite.as_view(),
        name='send-invite'),

    url(r'^accept-invite/(?P<key>\w+)/$', views.AcceptInvite.as_view(),
        name='accept-invite'),
    
)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
import django.utils.timezone


class Migration(migrations.Migration):

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Invitation',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('email', models.EmailField(unique=True, max_length=75, verbose_name='e-mail address')),
                ('accepted', models.BooleanField(default=False, verbose_name='accepted')),
                ('created', models.DateTimeField(default=django.utils.timezone.now, verbose_name='created')),
                ('key', models.CharField(unique=True, max_length=64, verbose_name='key')),
                ('sent', models.DateTimeField(null=True, verbose_name='sent')),
            ],
            options={
            },
            bases=(models.Model,),
        ),
    ]



from django.core.management.base import BaseCommand
from .. .models import Invitation

class Command(BaseCommand):
    def handle(self, *args, **options):
        Invitation.objects.delete_expired_confirmations()


from datetime import timedelta

from django.db import models
from django.utils import timezone
from django.db.models import Q

from . import app_settings


class InvitationManager(models.Manager):

    def all_expired(self):
        return self.filter(self.expired_q())

    def all_valid(self):
        return self.exclude(self.expired_q())

    def expired_q(self):
        sent_threshold = timezone.now() - timedelta(days=app_settings.INVITATION_EXPIRY)
        q = Q(accepted=True) | Q(sent__lt=sent_threshold)
        return q

    def delete_expired_confirmations(self):
        self.all_expired().delete()



from distutils.core import setup
from setuptools import setup, find_packages

setup(
  name = 'django-invitations',
  packages=find_packages(),
  package_data={'invitations': ['templates/*.*']},
  include_package_data=True,
  zip_safe=False,
  version = '0.12',
  description = 'Django invitation integration for django-allauth',
  author = 'https://github.com/bee-keeper',
  author_email = 'none@none.com',
  url = 'https://github.com/bee-keeper/django-invitations.git',
  download_url = 'https://github.com/bee-keeper/django-invitations/tarball/0.12',
  keywords = ['django', 'invitation', 'django-allauth', 'invite'],
  classifiers = [],
  install_requires=[
    'django-allauth>=0.18.0',
    'django-braces>=1.4.0',
  ],
)



from django.contrib import admin

from .models import Invitation


class InvitationAdmin(admin.ModelAdmin):
    list_display = ('email', 'sent', 'accepted')
    model = Invitation

admin.site.register(Invitation, InvitationAdmin)
